/* GObject - GLib Type, Object, Parameter and Signal Library
 * Copyright (C) 2000-2001 Tim Janik
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * gbsearcharray.h: binary searchable sorted array maintenance
 */
#ifndef __G_BSEARCH_ARRAY_H__
#define __G_BSEARCH_ARRAY_H__

#include        <glib/gtypes.h>
#include        <glib/gutils.h>
#include        <glib/gmem.h>
#include        <glib/gmessages.h>

G_BEGIN_DECLS

/* helper macro to avoid signed overflow for value comparisions */
#define	G_BSEARCH_ARRAY_CMP(v1,v2) ((v1) < (v2) ? -1 : (v1) > (v2))


/* --- typedefs --- */
typedef struct _GBSearchArray         GBSearchArray;
typedef gint  (*GBSearchCompareFunc) (gconstpointer bsearch_node1,
				      gconstpointer bsearch_node2);
typedef enum
{
  G_BSEARCH_ARRAY_ALIGN_POWER2	= 1 << 0,
  G_BSEARCH_ARRAY_DEFER_SHRINK	= 1 << 1
} GBSearchArrayFlags;


/* --- structures --- */
struct _GBSearchArray
{
  GBSearchCompareFunc cmp_nodes;
  guint16	      flags;
  guint16             sizeof_node;
  guint               n_nodes;
  gpointer            nodes;
};


/* --- prototypes --- */
GBSearchArray*	g_bsearch_array_new		(guint16	     sizeof_node,
						 GBSearchCompareFunc node_cmp_func,
						 GBSearchArrayFlags  flags);
void		g_bsearch_array_destroy		(GBSearchArray	*barray);
gpointer	g_bsearch_array_insert		(GBSearchArray	*barray,
						 gconstpointer	 key_node,
						 gboolean	 replace_existing);
void		g_bsearch_array_remove		(GBSearchArray	*barray,
						 gconstpointer	 key_node);
void		g_bsearch_array_remove_node	(GBSearchArray	*barray,
						 gpointer	 node_in_array);
/*douwen*/
gpointer	g_bsearch_array_lookup		(GBSearchArray	*barray,
						 gconstpointer	 key_node);

gpointer	g_bsearch_array_get_nth		(GBSearchArray	*barray,
						 guint		 n);

guint		g_bsearch_array_get_index	(GBSearchArray	*barray,
						 gpointer	 node_in_array);


/* initialization of static arrays */
#define	G_STATIC_BSEARCH_ARRAY_INIT(sizeof_node, cmp_nodes, flags) \
  { (cmp_nodes), (flags), (sizeof_node), 0, NULL }


/* --- implementation details --- */


G_END_DECLS

#endif /* __G_BSEARCH_ARRAY_H__ */
