/*
	KLibrary : kthread.c

	kernel thread library

	collected and modified by flyduck 2001/05/18
	
	original sources are from http://www.scs.ch/~frey/linux/
	Original writer is Martin Frey, frey@scs.ch
*/

#ifndef __KERNEL_LIBRARY_THREAD_H
#define __KERNEL_LIBRARY_THREAD_H

#include <linux/version.h>
#include <linux/sched.h>
#include <linux/tqueue.h>
#include <linux/wait.h>

#include <asm/semaphore.h>

//
// Type definitions
//

struct kthread_struct;

typedef int (*kthread_func_t)(struct kthread_struct *);

typedef struct kthread_struct {
	//
	// private data 
	//

	/* Linux task structure of thread */
	struct task_struct *thread;

	/* Task queue need to launch thread */
	struct tq_struct tq;

	/* function to be started as thread */
	kthread_func_t function;

	/* semaphore needed on start and creation of thread. */
	struct semaphore startstop_sem;

	//
	// public data
	//

	/* queue thread is waiting on. Gets initialized by
	   init_kthread, can be used by thread itself.
	*/
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)	
	wait_queue_head_t queue;
#else
	struct wait_queue *queue;
#endif	

	/* flag to tell thread whether to die or not.
	   When the thread receives a signal, it must check
	   the value of terminate and call exit_kthread and terminate
	   if set.
	*/
	int terminate;

	/* additional data to pass to kernel thread */
	void *param;
} kthread_t;

/* prototypes */

/* start new kthread (called by creator) */
void klib_start_kthread(kthread_func_t func, kthread_t *kthread, void *param);

/* stop a running thread (called by "killer") */
void klib_stop_kthread(kthread_t *kthread);

/* setup thread environment (called by new thread) */
void klib_init_kthread(kthread_t *kthread, char *name);

/* cleanup thread environment (called by thread upon receiving termination signal) */
void klib_exit_kthread(kthread_t *kthread);

#endif
