#ifndef __KORBIT_NETDB_H__
#define __KORBIT_NETDB_H__

#include <sys/socket.h>
#include <linux/in.h>
#include <arpa/inet.h>
#include "host_list.h"

#define h_errno 0
#define NI_MAXHOST      1025
#define NI_MAXSERV      32
/* static int h_errno; */

/* Description of data base entry for a single host.  */
struct hostent
{
  char *h_name;			/* Official name of host.  */
  char **h_aliases;		/* Alias list.  */
  int h_addrtype;		/* Host address type.  */
  socklen_t h_length;		/* Length of address.  */
  char **h_addr_list;		/* List of addresses from name server.  */
#define	h_addr	h_addr_list[0]	/* Address, for backward compatibility.  */
};

/* Is this defined somewhere else? */
/*struct in_addr {
	__uint32_t s_addr;
};*/

/*
 * Set h_errno?
 * #define HOST_NOT_FOUND 1
 * #define TRY_AGAIN 2
 * #define NO_RECOVERY 3
 * #define NO_DATA 4
 */
static inline struct hostent *gethostbyname (const char *host)
{
	int c;
	static struct in_addr tmp_in;
	static struct hostent ret_host;
	static char *aliases[2];
	static char *addrs[2];

	if (host == NULL)
		{
		printk ("** gethostbyname() Error: Got NULL parameter! **\n");
		return (NULL);
		}

	/*
	 * The actual lookup.
	 */
	for (c = 0; c < __MAX_STATIC_NAMES; c++)
		{
			if (host_table[c].name && strncmp (host, host_table[c].name, strlen(host_table[c].name)) == 0)
				{
/*				printk ("Name '%s' found at position %d!\n", argv[1], c);*/
/*				printk ("IP address is: '%s'.\n", IPs[c]);*/
				break;
				}
		}

	if (c == __MAX_STATIC_NAMES)
		{
		/* Host not found, return NULL. */
		return (NULL);
		}
	/* else, names[c] is gold! */

	/* make a new hostent, ret_host */

	ret_host.h_addrtype = AF_INET;
	ret_host.h_aliases = aliases;
	aliases[0] = host_table[c].name;
	aliases[1] = NULL;
	ret_host.h_name = host_table[c].name;
	if (!inet_aton (host_table[c].IP, &tmp_in))
		{
		printk ("** gethostbyname() Error: Invalid IP address in table! **\n");
		return (NULL);
		}
	ret_host.h_addr_list = addrs;
	addrs[0] = (char *)&tmp_in.s_addr;
	addrs[1] = NULL;
	ret_host.h_length = sizeof (tmp_in.s_addr);
	return (&ret_host);
} /* End gethostbyname(). */

/*
 * TODO: getpeername(), gethostbyaddr(), getsockname(), gethostname()
 * Everything from here-on has been untested (in userland). Buyer beware.
 */
static inline struct hostent *gethostbyaddr (const char *addr, int len, int type)
{
	struct in_addr tin;
	char *inp_addr;
	int c;
	static struct hostent ret_host;
	static char *aliases[1];
	static char *addrs[2];
	static struct in_addr tmp_in;


	if ((type != AF_INET) || (len != 4))
		{
		printk ("** gethostbyaddr Error: Don't know how to deal with non-AF_INET addresses! **\n");
		return (NULL);
		}

	tin.s_addr = *((__u32 *)addr);
	inp_addr = inet_ntoa (tin);
	if (inp_addr == NULL)
		{
		/* We got some invalid input, baby. */
		return (NULL);

		}

	/*
	 * The actual lookup.
	 */
	for (c = 0; c < __MAX_STATIC_NAMES; c++)
		{
			if (host_table[c].IP && strncmp (inp_addr, host_table[c].IP, strlen(host_table[c].IP)) == 0)
				{
				break;
				}
		}

	if (c == __MAX_STATIC_NAMES)
		{
		/* Host not found, return NULL. */
		return (NULL);
		}
	/* else, host_table[c].IP is gold! */

	ret_host.h_addrtype = AF_INET;
	ret_host.h_aliases = aliases;
	aliases[0] = NULL;
	ret_host.h_name = host_table[c].name;
	if (!inet_aton (host_table[c].IP, &tmp_in))
		{
		printk ("** gethostbyname() Error: Invalid IP address in table! **\n");
		return (NULL);
		}
	ret_host.h_addr_list = addrs;
	addrs[0] = (char *)&tmp_in.s_addr;
	addrs[1] = NULL;
	ret_host.h_length = sizeof (tmp_in.s_addr);
	return (&ret_host);	
} /* End gethostbyaddr(). */


#endif
