#ifndef __KORBIT_STDIO_H__
#define __KORBIT_STDIO_H__

#include <asm/segment.h>
#include <asm/uaccess.h>
#include <linux/smp_lock.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/sched.h>
#include <linux/uio.h>
#include <linux/dcache.h>
#include <linux/file.h>
#include <linux/highuid.h>
#include <sys/socket.h>

#include <errno.h>
#define  fgets(x,y,z) 
#define fputs(device,msg)
#define printf(format,args...) printk(format,##args)
#define fprintf(device,format,args...) printk(format,##args)
#define fflush(x)

#define SEEK_SET       0       /* Seek from beginning of file.  */
#define SEEK_CUR       1       /* Seek from current position.  */
#define SEEK_END       2       /* Seek from end of file.  */

static inline int unlink(const char *pathname) {
  printf("KERNEL UNLINK('%s') CALLED!\n", pathname);
  return -1;
}

static inline struct file *fd2file(int fd) {
        if (fd & 1) /* can't convert a socket! */
                return NULL;

        return (struct file *)(-(fd & ~1));
}

static inline int open(const char *filename, int flags, int mode) {
        struct file *RetVal = filp_open(filename, flags, mode);
        if (IS_ERR(RetVal))
        {
                errno = PTR_ERR(RetVal);
                return -1;
        }
        return -(int)RetVal;
}

static inline int creat(const char *filename, mode_t mode)
{
        return open(filename, O_CREAT | O_WRONLY | O_TRUNC, mode);
}

static inline int lseek(int fd, long offset, int whence)
{
        if ((fd & 1) == 1)
        {
                return -1;
        }
        else 
        {
                struct file *F = fd2file(fd);
                loff_t (*fn)(struct file *, loff_t, int);
                int retval = -1;

                if (whence <= 2) 
                {
                        fn = default_llseek;
                        if (F->f_op && F->f_op->llseek)
                                fn = F->f_op->llseek;
                        
                        lock_kernel();
                        retval = fn(F, offset, whence);
                        unlock_kernel();
                }
                if (retval < 0)
                {
                        errno = -retval;
                        retval = -1;
                }
                return retval;
        }
}


asmlinkage long sys_newstat(char * filename, struct stat * statbuf);
static inline int stat(char *filename, struct stat *buf)
{
        mm_segment_t oldfs;
        int retval;
        
        oldfs = get_fs(); set_fs(KERNEL_DS); 
        retval = sys_newstat(filename, buf);
        set_fs(oldfs);
        if (retval < 0)
        {
                errno = -retval;
                retval = -1;
        }
        return retval;
}

asmlinkage long sys_newlstat(char * filename, struct stat * statbuf);
static inline int lstat(char *filename, struct stat *buf)
{
        mm_segment_t oldfs;
        int retval;
        
        oldfs = get_fs(); set_fs(KERNEL_DS); 
        retval = sys_newlstat(filename, buf);
        set_fs(oldfs);
        if (retval < 0)
        {
                errno = -retval;
                retval = -1;
        }
        return retval;
}


static inline int do_revalidate(struct dentry *dentry)
{
        struct inode * inode = dentry->d_inode;
        if (inode->i_op && inode->i_op->revalidate)
                return inode->i_op->revalidate(dentry);
        return 0;
}


static inline int cp_new_stat(struct inode * inode, struct stat * statbuf)
{
        struct stat tmp;
        
        memset(&tmp, 0, sizeof(tmp));
        tmp.st_dev = kdev_t_to_nr(inode->i_dev);
        tmp.st_ino = inode->i_ino;
        tmp.st_mode = inode->i_mode;
        tmp.st_nlink = inode->i_nlink;
        SET_STAT_UID(tmp, inode->i_uid);
        SET_STAT_GID(tmp, inode->i_gid);
        tmp.st_rdev = kdev_t_to_nr(inode->i_rdev);
#if BITS_PER_LONG == 32
        if (inode->i_size > 0x7fffffff)
                return -EOVERFLOW;
        else
#endif  
        {
                tmp.st_size = inode->i_size;
                tmp.st_atime = inode->i_atime;
                tmp.st_mtime = inode->i_mtime;
                tmp.st_ctime = inode->i_ctime;
                
                tmp.st_blocks = inode->i_blocks;
                tmp.st_blksize = inode->i_blksize;
                
                memcpy(statbuf, &tmp, sizeof(tmp));
                return 0;
        }
}


static inline int fstat(int fd, struct stat *buf)
{
        if ((fd & 1) == 1)
        {
                errno = EBADF;
                return -1;
        }
        else
        {
                struct file *file = fd2file(fd);
                struct dentry *dentry;
                int retval = -EBADF;
                
                if (file) 
                {
                        dentry = file->f_dentry;
                        retval = do_revalidate(dentry);
                        
                        if (!retval)
                                retval =  cp_new_stat(dentry->d_inode, buf);
                }
                if (retval < 0)
                {
                        errno = -retval;
                        retval = -1;
                }
                return retval;
        }
}


asmlinkage long sys_readlink(const char * path, char * buf, int bufsiz);
static inline int readlink(const char *path, char *buf, size_t bufsiz)
{
        int retval;
        mm_segment_t oldfs;
        
        oldfs = get_fs(); set_fs(KERNEL_DS); 
        retval = sys_readlink(path, buf, bufsiz);
        set_fs(oldfs);
        if (retval < 0)
        {
                errno = -retval;
                retval = -1;
        }
        return retval;
}


static inline int read(int fd, void *buffer, size_t count) {
        if ((fd & 1) == 1)
        {
                struct socket *sock = fd2sock(fd);
                struct iovec iov;
                struct msghdr msg;
		mm_segment_t oldfs;
                int flags = 0;
                int RetVal;
                
                msg.msg_name = NULL;
                msg.msg_namelen = 0;
                msg.msg_iov = &iov;
                msg.msg_iovlen = 1;
                msg.msg_control = NULL;
                msg.msg_controllen = 0;
		msg.msg_flags = 0;
                iov.iov_base = buffer;
                iov.iov_len = count;
		oldfs = get_fs(); set_fs(KERNEL_DS); 
                RetVal = sock_recvmsg(sock, &msg, count, flags);
		set_fs(oldfs);
                if (RetVal < 0)
                {
                        errno = -RetVal;
                        RetVal = -1;
                }
                return RetVal;
        }
        else
        {
                struct file *F = fd2file(fd);
                mm_segment_t oldfs;
                int RetVal;
                
                oldfs = get_fs(); 
                set_fs(KERNEL_DS);
                RetVal = F->f_op->read(F, buffer, count, &F->f_pos);
                set_fs(oldfs);
                if (RetVal < 0)
                {
                        errno = -RetVal;
                        RetVal = -1;
                }
                return RetVal;
        }
}

static inline int write(int fd, const void *buffer, size_t count) {
        if ((fd & 1) == 1)
        {
                struct socket *sock = fd2sock(fd);
                struct iovec iov;
                struct msghdr msg;
		mm_segment_t oldfs;
                int RetVal;
                msg.msg_name = NULL;
                msg.msg_namelen = 0;
                msg.msg_iov = &iov;
                msg.msg_iovlen = 1;
                msg.msg_control = NULL;
                msg.msg_controllen = 0;
		msg.msg_flags = MSG_NOSIGNAL;
                if (sock->type == SOCK_SEQPACKET)
                        msg.msg_flags |= MSG_EOR;
                iov.iov_base = (void *)buffer;
                iov.iov_len = count;

		oldfs = get_fs(); set_fs(KERNEL_DS); 
                RetVal = sock_sendmsg(sock, &msg, count);
		set_fs(oldfs);
                if (RetVal < 0)
                {
                        errno = -RetVal;
                        RetVal = -1;
                }
                return RetVal;
        }
        else
        {
                struct file *F = fd2file(fd);
                mm_segment_t oldfs;
                int RetVal;
                
                oldfs = get_fs(); 
                set_fs(KERNEL_DS);
                RetVal = F->f_op->write(F, buffer, count, &F->f_pos);
                set_fs(oldfs);
                if (RetVal < 0)
                {
                        errno = -RetVal;
                        RetVal = -1;
                }
                return RetVal;
        }
}

static inline int writev(int fd, const struct iovec  *vector, int count) {
#ifndef DONT_USE_SIMPLE_WRITEV
  int i, amount = 0;
  for (i = 0; i < count; i++) {
    char *Buf = vector[i].iov_base;
    int Amount = vector[i].iov_len;
    while (Amount > 0) {
      int A = write(fd, Buf, Amount);
      Amount -= A;
      amount += A;
      Buf += A;
      if (Amount > 0) schedule();     // Behave somewhat nicely...
    }
  }

  return amount;

#else
        if ((fd & 1) == 1)
        {
                struct socket *sock =  fd2sock(fd);
                struct msghdr msg;
		mm_segment_t oldfs;
                int i, RetVal;
                size_t tot_len = 0;
                
                for (i = 0; i < count; i++)
                        tot_len += vector[i].iov_len;
                
                msg.msg_name = NULL;
                msg.msg_namelen = 0;
                msg.msg_iov = (struct iovec *)vector;
                msg.msg_iovlen = count;
                msg.msg_control = NULL;
                msg.msg_controllen = 0;
                if (sock->type == SOCK_SEQPACKET)
                        msg.msg_flags |= MSG_EOR;

		oldfs = get_fs(); set_fs(KERNEL_DS); 
                RetVal = sock_sendmsg(sock, &msg, tot_len);
		set_fs(oldfs);
                if (RetVal < 0)
                {
                        errno = -RetVal;
                        RetVal = -1;
                }
                return RetVal;
        }
        else
        {
                struct file *F = fd2file(fd);
                mm_segment_t oldfs;
                int RetVal;
                
                oldfs = get_fs(); 
                set_fs(KERNEL_DS);
                RetVal = F->f_op->writev(F, vector, (unsigned)count, &F->f_pos);
                set_fs(oldfs);
                if (RetVal < 0)
                {
                        errno = -RetVal;
                        RetVal = -1;
                }
                return RetVal;
        }
#endif
}

static inline int close(int fd) {
        int err = 0;
        if ((fd & 1) == 1) {
                struct socket *sock = fd2sock(fd);
                sock_release(sock);
        } else {
                struct file *file = fd2file(fd);
                fput(file);
        }
        return err;
}

#endif
