#ifndef __KORBIT_STDLIB_H__
#define __KORBIT_STDLIB_H__

#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/types.h>
#include <asm/string.h>

#define strtol(nptr,endptr,base) simple_strtol(nptr,endptr,base)

#if 0
#define malloc(size) kmalloc(size, GFP_KERNEL)
#define free(ptr) kfree(ptr)
#endif

static inline void *malloc(size_t size)
{
        void *ptr = NULL;

        if (size == 0) 
                size = 4;

        ptr = kmalloc(size + sizeof(size), GFP_KERNEL);

        if (ptr)
        {
                *((size_t *)ptr) = size;
                ptr = (size_t *)ptr + 1;
        }
        
        return ptr;
}

static inline void free(void *ptr)
{
        if (ptr)
                kfree((size_t *)ptr - 1);
}

#define alloca(size) malloc(size)

/* freeca(ptr) - free a mem allocation if ptr points to one, otherwise do 
 * nothing.
 */
static inline void freeca(void *ptr)
{
    if (ptr != NULL) { /* Don't free it if it's already free */
	free(ptr);
    }
}

static inline void *calloc(size_t nmemb, size_t size)
{
        void *ptr = malloc(nmemb*size);
        if (ptr)
                memset(ptr,0,nmemb*size);
        return ptr;
}

static inline void *realloc(void *ptr, size_t size)
{
        void *newptr = NULL;

        if (size != 0)
                newptr = malloc(size);
        
        if (ptr && newptr) /* Copy old contents */
        {
                size_t *p1 = (size_t *)ptr - 1;
                size_t *p2 = (size_t *)newptr - 1;
                size_t n = *p1 < *p2 ? *p1 : *p2;
                memcpy(newptr, ptr, n);
        }

        if (ptr)
                free(ptr);
        
        return newptr;
}

/* Returned by `div'.  */
typedef struct
{
        int quot;			/* Quotient.  */
        int rem;			/* Remainder.  */
} div_t;

static inline div_t div(int number, int denom)
{
        div_t result;
        result.quot = number/denom;
        result.rem = number-(number*result.quot);
        return result;
}

#define atexit(fn) -1
#define getenv(name) 0

#endif /* __KORBIT_STDLIB_H__ */
