#ifndef __KORBIT_SYS_POLL_H__
#define __KORBIT_SYS_POLL_H__

#include <asm/poll.h>
#include <asm/param.h>
#include <linux/net.h>
#include <linux/tcp.h>
#include <linux/socket.h>
#include <net/tcp.h>
#include <net/sock.h>
#include <linux/skbuff.h>
#include <linux/sched.h>
#include "stdlib.h"
#include "sys/socket.h"

/* Poll the file descriptors described by the NFDS structures starting at
 * FDS.  If TIMEOUT is nonzero and not -1, allow TIMEOUT milliseconds for
 * an event to occur; if TIMEOUT is -1, block until an event occurs.
 * Returns the number of file descriptors with events, zero if timed
 * out, or -1 for errors.
 */

/* This implementation of poll assumes that we are sitting on the wait
 * queues for all of the file descriptors already.  Therefore if we are about
 * to block, all we have to do is perform the schedule call which will
 * automatically cause us to "block".
 */
static int poll_CheckFDs(struct pollfd *fds, unsigned long int nfds)
{
  int NumReady = 0, i;

  // Loop over the file descriptors seeing if there is already work to be
  // done...
  for (i = 0; i < nfds; i++) {
        struct socket *sock = fd2sock(fds[i].fd);
	fds[i].revents = 0;

        // Check to see if stuff is available to read
        if (sock) {
                // It's a socket baby
                /*fds[i].revents = tcp_poll(0, sock, 0);douwen*/

                // Apparently tcp_poll doesn't look at the CLOSE_WAIT value,
                // and we have a lot of sockets that end up in this state.
                // This is a hack to shortcircuit some read failures from
                // later.  This breaks "poll semantics" strictly speaking, but
                // it's basically the "right thing to do" (tm).
		if (sock->sk->state == TCP_CLOSE_WAIT) 
                        fds[i].revents = POLLHUP;
               	fds[i].revents &= fds[i].events | POLLERR | POLLHUP;
        } else {
                // It's a file
                //struct file *f = fd2file(fd);
                BUG();
	}

        if (fds[i].revents) {
                NumReady++;
	}

  } /* for */

  return NumReady;
} /* End poll_CheckFDs(). */


static int poll(struct pollfd *fds, unsigned long int nfds, int timeout) {
  wait_queue_t *WaitQueues = 0;
  int NumReady = poll_CheckFDs(fds, nfds);
  int i; 

  if (NumReady || timeout == 0)
    return NumReady;

  WaitQueues = (wait_queue_t*)malloc(nfds*sizeof(wait_queue_t));
  if (WaitQueues == 0) return 0; // Crap, nomem...

  for (i = 0; i < nfds; i++) { 
    struct socket *sock = fd2sock(fds[i].fd);
    init_waitqueue_entry(WaitQueues+i, current);

    if (sock)
      add_wait_queue_exclusive(sock->sk->sleep, WaitQueues+i);
  }

  // Wait for us to get notified by one of the socket wait queue notifiers!
  do {
    // Change our task state so that we are not immediately rescheduled.  
    // This lets the scheduler know that we are waiting for something to happen
    set_current_state(TASK_INTERRUPTIBLE);
    schedule();
    if ( signal_pending(current) )
       break;
  } while (!(NumReady = poll_CheckFDs(fds, nfds)));

  set_current_state(TASK_RUNNING);

  for (i = 0; i < nfds; i++) { 
    struct socket *sock = fd2sock(fds[i].fd);
    if (sock)
      remove_wait_queue(sock->sk->sleep, WaitQueues+i);
  }

  free(WaitQueues);
  return NumReady;
}

#endif
