#ifndef __KORBIT_SYS_SOCKET_H__
#define __KORBIT_SYS_SOCKET_H__
typedef unsigned int socklen_t;

#include <linux/socket.h>
#include <linux/wait.h>
#include <asm/semaphore.h>
#include <net/sock.h>

/* These functions are declared in net/socket.c */
/*asmlinkage long sys_socket(int family, int type, int protocol);*/
struct socket *sockfd_lookup(int fd, int *err);


static inline int sock2fd(struct socket *s)
{
        return (-(int)s) | 1;
}

static inline struct socket *fd2sock(int sockfd)
{
        if ((sockfd & 1) == 0) /* can't convert a file! */
                return NULL;

        return (struct socket *)(-(sockfd & ~1));
}

#ifdef NOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO
static inline int socket(int domain, int type, int protocol) {
        struct socket *sock;
        int retval = sock_create(domain, type, protocol, &sock);

        if (retval < 0) return (int)NULL;
        return sock2fd(sock);
}


static inline int bind(int sockfd, struct sockaddr *my_addr, socklen_t addrlen)
{
        struct socket *sock = fd2sock(sockfd);

        if (sock == NULL)
                return -1;
        if (!sock->ops->bind(sock, my_addr, addrlen))
                return 0;
        else
                return -1; /* should normally also set errno */
}


static inline int connect(int sockfd, const struct sockaddr *serv_addr, 
                          socklen_t addrlen)
{
        struct socket *sock = fd2sock(sockfd);
        int flags = 0; /* TODO : what is flags supposed to be? */
        
        if (sock == NULL)
                return -1;

        if (sock->ops->connect(sock, (struct sockaddr *)serv_addr, addrlen, flags) == 0)
                return 0;
        else
                return -1; /* should normally also set errno */
}


static inline int listen(int sockfd, int backlog)
{
        struct socket *sock = fd2sock(sockfd);
        
        if (sock == NULL)
                return -1;
        
        if (sock->ops->listen(sock, backlog) == 0)
                return 0;
        else
                return -1; /* should normally also set errno */
}


static inline int accept(int sockfd, struct sockaddr *addr, socklen_t *addrlen)
{
        struct socket *sock = fd2sock(sockfd);
        struct socket *newsock;
        struct sockaddr_in *newaddr = (struct sockaddr_in *)addr; /* check? */
        int flags = 0; /* Should be ok */

	if (sock == 0) 
                return -1;
        
        newsock = sock_alloc();
        if (newsock == 0)
                return -1;

        newsock->type = sock->type;
        newsock->ops = sock->ops;
        if (sock->ops->accept(sock, newsock, flags) < 0) 
        {
                sock_release(newsock);
                return -1; /* should normally also set errno */
        }

        newaddr->sin_family = AF_INET;
        newaddr->sin_port = newsock->sk->dport;
        newaddr->sin_addr.s_addr = newsock->sk->daddr;

        *addrlen = sizeof(newaddr);

        return sock2fd(newsock);
}


static inline int shutdown(int sockfd, int how)
{
        struct socket *sock = fd2sock(sockfd);
        
        if (sock == NULL)
                return -1;

        if (sock->ops->shutdown(sock, how) == 0)
                return 0;
        else
                return -1; /* should normally also set errno */
}
#endif /*NOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO*/
#endif
